/*
 * Copyright (c) 2019.
 * Autor: Beatriz Domínguez Callero
 * Esta obra está sujeta a una licencia de ReconocimientoNoComercial-SinObraDerivada 3.0 España de Creative Commons:
 * https://creativecommons.org/licenses/by-nc-nd/3.0/es/
 */

package com.beatriz.gestionaparcamiento.controller.activities;

import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.CalendarView;
import android.widget.FrameLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.beatriz.gestionaparcamiento.model.api.ApiCalls;
import com.beatriz.gestionaparcamiento.model.api.interfaces.DeleteBookingParkingCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.DeleteReleaseParkingCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.GetBookingCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.GetInfoReleaseCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.GetParkingCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.GetUserParkingCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.SaveReleaseParkingCallBack;
import com.beatriz.gestionaparcamiento.model.objects.Booking;
import com.beatriz.gestionaparcamiento.model.objects.Parking;
import com.beatriz.gestionaparcamiento.model.objects.User;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.FragmentTransaction;

import com.beatriz.gestionaparcamiento.R;

import java.util.Date;

import com.beatriz.gestionaparcamiento.controller.fragments.MyParkingFragment;
import com.beatriz.gestionaparcamiento.controller.fragments.ParkingFreeFragment;
import com.beatriz.gestionaparcamiento.model.preferences.SharedPreference;
import com.beatriz.gestionaparcamiento.controller.utils.Utils;

public class BookingActivity extends AppCompatActivity {

    User user;
    Parking parking;
    FrameLayout LL_popUp;
    FragmentTransaction fragment_transation;
    MyParkingFragment fragment_MyParking;
    ParkingFreeFragment parkingFreeFragment;
    Date date;
    //Booking content_
    Button BT_calendar;
    TextView TV_info;
    Button BT_release;
    Button BT_booking;
    RelativeLayout RL_progress_bar;


    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_booking);
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        //Inicializa la vista de booking
        BT_calendar  = findViewById(R.id.BT_calendar);
        TV_info = findViewById(R.id.TV_info);
        BT_release = findViewById(R.id.BT_release);
        BT_booking = findViewById(R.id.BT_booking);
        date = Utils.getDateToday();
        initBTCalendar();
        RL_progress_bar = findViewById(R.id.RL_progress_bar);
        boolean load_booking = true;

        //Inicializa el fragment my parking si es necesario
        LL_popUp = findViewById(R.id.layout_popUp);
        user = (User) getIntent().getSerializableExtra("user");
        if(user.getParking_id()==null){
            load_booking=false;
            getUserParkingFromServer();
        }else{
            parking=SharedPreference.getInstance(getApplicationContext()).getParkingPreferences();
        }

        //Carga datos
        if(load_booking){
            loadData();
        }

    }


    /**************** BOOKING PARKING METHODS *******************/

    private void initBTCalendar(){
        BT_calendar.setText(Utils.calendarToUIdate(date));
        BT_calendar.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                final RelativeLayout RL_calendar = findViewById(R.id.RL_calendar);
                RL_calendar.setVisibility(View.VISIBLE);
                RL_calendar.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        RL_calendar.setVisibility(View.INVISIBLE);
                    }
                });
                CalendarView calendarView = findViewById(R.id.calendarView);
                calendarView.setOnDateChangeListener(new CalendarView.OnDateChangeListener() {
                    @Override
                    public void onSelectedDayChange(@NonNull CalendarView view, int year, int month, int dayOfMonth) {
                        RL_calendar.setVisibility(View.INVISIBLE);
                        date=Utils.createDate(year,month,dayOfMonth);
                        BT_calendar.setText(Utils.calendarToUIdate(date));
                        loadData();
                    }
                });



            }
        });
    }

    private void loadData(){
        //Carga los datos necesarios para la interfaz: la información de las liberaciones de plaza y las reservas del usuario
        RL_progress_bar.setVisibility(View.VISIBLE);
        final String date_bbdd = Utils.calendarToBBDDdate(date);
        ApiCalls.getInstance(getApplicationContext()).get_info_release(date_bbdd, user.getParking_id(), new GetInfoReleaseCallBack() {
            @Override
            public void onCallback(final Booking release, String error) {
                ApiCalls.getInstance(getApplicationContext()).get_booking(date_bbdd, user.getUser_id(), new GetBookingCallBack() {
                    @Override
                    public void onCallback(Booking booking, String error) {
                        if (booking == null & release == null) {
                            loadBooking1();
                        }else if (booking == null & release != null){
                            loadBooking2(release);
                        }else if (booking!=null & release!=null){
                            loadBooking3(booking);
                        }else if(booking!=null & release==null){
                            loadBooking4(booking);
                        }
                        RL_progress_bar.setVisibility(View.INVISIBLE);
                    }
                });
            }
        });
    }
    private void loadBooking1(){
        //Carga la interfaz del estado 1 de reservas
        TV_info.setText(getString(R.string.booking_text1).replace("XXX",user.getParking_id().toString()));
        BT_release.setBackground(getDrawable(R.drawable.rounded_corner_color_primary));
        BT_release.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                releaseMyParking(parking);
            }
        });
        BT_booking.setBackground(getDrawable(R.drawable.rounded_corner_color_primary));
        BT_booking.setText(getString(R.string.booking_bt_booking_other));
        BT_booking.setOnClickListener(
                new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                showMyFreeParkingFragment();
            }
        });
    }
    private void loadBooking2(final Booking release){
        //Carga la interfaz del estado 2 de reservas
        TV_info.setText(getString(R.string.booking_text2));
        BT_release.setBackground(getDrawable(R.drawable.rounded_corner_color_primary_inactive));
        BT_booking.setBackground(getDrawable(R.drawable.rounded_corner_color_primary));
        BT_release.setOnClickListener(null);
        BT_booking.setText(getString(R.string.booking_bt_booking));
        BT_booking.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(release.getUser_id()==null){
                    //Puedo reservar mi parking liberado anteriormente
                    deleteReleaseMyParking();
                }else{
                    //Tengo que reservar otra
                    showMyFreeParkingFragment();
                }
            }
        });
    }
    private void loadBooking3(final Booking booking){
        //Carga la interfaz del estado 3 de reservas
        TV_info.setText(getString(R.string.booking_text3).replace("XXX",booking.getParkingId().toString()));
        BT_release.setBackground(getDrawable(R.drawable.rounded_corner_color_primary));
        BT_release.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Parking parking_to_release = new Parking(booking.getParkingId(),booking.getAdmit_big());
                releaseAndDeleteOtherParking(parking_to_release);
            }
        });
        BT_booking.setBackground(getDrawable(R.drawable.rounded_corner_color_primary_inactive));
        BT_booking.setOnClickListener(null);
        BT_booking.setText(getString(R.string.booking_bt_booking));
    }
    private void loadBooking4(final Booking booking){
        //Carga la interfaz del estado 4 de reservas
        TV_info.setText(getString(R.string.booking_text4).replace("YYY",user.getParking_id().toString()).replace("XXX",booking.getParkingId().toString()));
        BT_release.setBackground(getDrawable(R.drawable.rounded_corner_color_primary));
        BT_release.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Parking parking_to_release = new Parking(booking.getParkingId(),booking.getAdmit_big());
                releaseAndDeleteOtherParking(parking_to_release);
            }
        });
        BT_booking.setBackground(getDrawable(R.drawable.rounded_corner_color_primary_inactive));
        BT_booking.setOnClickListener(null);
        BT_booking.setText(getString(R.string.booking_bt_booking));
    }
    private void releaseAndDeleteOtherParking(final Parking  parking_to_release){
        //Cancela la reserva del parking
        RL_progress_bar.setVisibility(View.VISIBLE);
        ApiCalls.getInstance(getApplicationContext()).delete_booking_parking(Utils.calendarToBBDDdate(date), user.getUser_id(), parking_to_release.getParking_id(), new DeleteBookingParkingCallBack() {
            @Override
            public void onCallback(String error) {
                if(error==null){
                    ApiCalls.getInstance(getApplicationContext()).release_parking(Utils.calendarToBBDDdate(date), parking_to_release, new SaveReleaseParkingCallBack() {
                        @Override
                        public void onCallback(String error) {
                            if(error==null){
                                loadData();
                            }else{
                                showMessages(error);
                            }
                            RL_progress_bar.setVisibility(View.INVISIBLE);
                        }
                    });
                }else{
                    showMessages(error);
                }
            }
        });

    }

    private void releaseMyParking(Parking parking_to_release){
        //Libera el parking de un usuario
        RL_progress_bar.setVisibility(View.VISIBLE);
        ApiCalls.getInstance(getApplicationContext()).release_parking(Utils.calendarToBBDDdate(date), parking_to_release, new SaveReleaseParkingCallBack() {
            @Override
            public void onCallback(String error) {
                if(error==null){
                    loadData();
                }else{
                    showMessages(error);
                }
                RL_progress_bar.setVisibility(View.INVISIBLE);
            }
        });
    }

    private void deleteReleaseMyParking(){
        //Cancela la liberacion de una plaza del usuario
        RL_progress_bar.setVisibility(View.VISIBLE);
        ApiCalls.getInstance(getApplicationContext()).delete_release_parking(Utils.calendarToBBDDdate(date), user.getParking_id(), new DeleteReleaseParkingCallBack() {
            @Override
            public void onCallback(String error) {
                if (error!=null){
                    showMessages(error);
                }else{
                    loadData();
                }
            }
        });
    }

    /**************** LOAD PARKING METHODS *******************/

    private void getUserParkingFromServer(){
        //Carga el parking del usuario
        RL_progress_bar.setVisibility(View.VISIBLE);
        ApiCalls.getInstance(getApplicationContext()).get_user_parking(user.getUser_id(), new GetUserParkingCallBack() {
            @Override
            public void onCallback(Integer value, String error) {
                RL_progress_bar.setVisibility(View.INVISIBLE);
                if(error!=null){
                    showMessages(error);
                }else {
                    user.setParking_id(value);
                    if(user.getParking_id()==null){
                        showMyParkingFragment();
                    }else{
                        getParkingFromServer();
                    }
                }
            }
        });
    }

    private void getParkingFromServer() {
        //Carga la info del parking del usuario
        RL_progress_bar.setVisibility(View.VISIBLE);
        ApiCalls.getInstance(getApplicationContext()).get_parking(user.getParking_id(), new GetParkingCallBack() {
            @Override
            public void onCallback(Parking value, String error) {
                RL_progress_bar.setVisibility(View.INVISIBLE);
                parking=value;
                if(error!=null){
                    showMessages(error);
                }else {
                    SharedPreference.getInstance(getApplicationContext()).setUserPreferences(user);
                    SharedPreference.getInstance(getApplicationContext()).setParkingPreferences(parking);
                    loadData();
                }
            }
        });
    }



    private void showMyParkingFragment(){
        //Muestra el fragment myparking para que el usuario pueda elegir su plaza.
        LL_popUp.setVisibility(View.VISIBLE);
        fragment_MyParking = MyParkingFragment.newInstance(user);
        fragment_transation= getSupportFragmentManager().beginTransaction();
        fragment_transation.replace(R.id.layout_popUp, fragment_MyParking);
        fragment_transation.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
        fragment_transation.commit();
    }


    public void saveUserWithParkingAndCloseFragment(User user_with_parking, boolean admit_big){
        //Recoge la informacion del fragment myparking
        SharedPreference.getInstance(getApplicationContext()).setUserPreferences(user_with_parking);
        parking = new Parking(user_with_parking.getParking_id(),admit_big);
        SharedPreference.getInstance(getApplicationContext()).setParkingPreferences(parking);
        fragment_transation.remove(fragment_MyParking);
        fragment_transation.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_CLOSE);
        LL_popUp.setVisibility(View.INVISIBLE);
        user=user_with_parking;
        loadData();
    }


    /**************** FREE PARKING METHODS *******************/

    private void showMyFreeParkingFragment(){
        //Muestra el fragment en el que se muestran las plazas disponibles para cierto dia para un usuaio
        LL_popUp.setVisibility(View.VISIBLE);
        parkingFreeFragment = ParkingFreeFragment.newInstance(Utils.calendarToBBDDdate(date),user);
        fragment_transation= getSupportFragmentManager().beginTransaction();
        fragment_transation.replace(R.id.layout_popUp, parkingFreeFragment);
        fragment_transation.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
        fragment_transation.commit();
    }

    public void closeFreeFragment(boolean load_data){
        //Recoge la información del fragment de plazas libres y la trata
        fragment_transation.remove(parkingFreeFragment);
        fragment_transation.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_CLOSE);
        LL_popUp.setVisibility(View.INVISIBLE);
        if(load_data)
            loadData();
    }

    /************* MENU**********************************/
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_booking, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Segun que opción del menu se seleccione, realiza una u otra acción
        switch (item.getItemId()) {
            case R.id.menu_my_parking:
                showMessages(getString(R.string.menu_my_parking_alert));
                return true;
            case R.id.menu_my_count:
                showMessages(getString(R.string.menu_my_count_alert));
                return true;
            case R.id.menu_close_session:
                SharedPreference.getInstance(getApplicationContext()).setUserPreferences(null);
                SharedPreference.getInstance(getApplicationContext()).setParkingPreferences(null);
                finish();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    /**************** OTHERS METHODS *******************/

    private void showMessages( String string_to_show) {
        //Muestra mensajes
        Toast.makeText(getApplicationContext(), string_to_show, Toast.LENGTH_SHORT).show();
    }

    @Override
    public void onBackPressed() {
        //Cuando se pulsa el botón de atrás, forzamos que la aplicación termine ya que la pantalla anterior es la de login
        //Y el usuario está almacenado en preferencias
        Intent homeIntent = new Intent(Intent.ACTION_MAIN);
        homeIntent.addCategory( Intent.CATEGORY_HOME );
        homeIntent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivity(homeIntent);
    }
}
