/*
 * Copyright (c) 2019.
 * Autor: Beatriz Domínguez Callero
 * Esta obra está sujeta a una licencia de ReconocimientoNoComercial-SinObraDerivada 3.0 España de Creative Commons:
 * https://creativecommons.org/licenses/by-nc-nd/3.0/es/
 */

package com.beatriz.gestionaparcamiento.controller.activities;
import android.content.Intent;
import android.os.Bundle;

import androidx.appcompat.app.AppCompatActivity;

import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.beatriz.gestionaparcamiento.R;
import com.beatriz.gestionaparcamiento.model.api.ApiCalls;
import com.beatriz.gestionaparcamiento.model.api.interfaces.LoginCallBack;
import com.beatriz.gestionaparcamiento.model.objects.User;

import com.beatriz.gestionaparcamiento.model.preferences.SharedPreference;
import com.beatriz.gestionaparcamiento.controller.utils.Utils;


public class LoginActivity extends AppCompatActivity {

    Button BT_register;
    Button BT_login;
    TextView TV_forgot_pass;
    EditText ET_email;
    EditText ET_password;
    RelativeLayout RL_progress_bar;


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);

        //Inicialicializamos los elementos:
        BT_register = findViewById(R.id.BT_register);
        BT_login = findViewById(R.id.BT_login);
        TV_forgot_pass = findViewById(R.id.TV_forgot_pass);
        ET_email = findViewById(R.id.ET_email);
        ET_password = findViewById(R.id.ET_password);
        RL_progress_bar = findViewById(R.id.RL_progress_bar);

        BT_register.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                pressRegister();
            }
        });

        BT_login.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                pressLogin();
            }
        });

        TV_forgot_pass.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                pressForgotPassword();
            }
        });

        checkIfUserInPreferences();
    }

    public void pressRegister(){
        Intent intent = new Intent(this, RegisterActivity.class);
        startActivityForResult(intent, 1);
    }

    public void pressLogin(){
        //Comprobamos si el email y la contraseña son válidos
        final String email = ET_email.getText().toString();
        final String password = ET_password.getText().toString();
        if(email.isEmpty()){
            ET_email.setError(getString(R.string.login_error_email_empty));
        }else if(!Utils.isEmailValid(email)){
            ET_email.setError(getString(R.string.login_error_email_invalid));
        }else if(password.isEmpty()){
            ET_password.setError(getString(R.string.login_error_password_empty));
        }
        else{
            //Si son válidos, lanzamos la llamada al API
            RL_progress_bar.setVisibility(View.VISIBLE);


            ApiCalls.getInstance(getApplicationContext()).login(email, password, new LoginCallBack() {
                @Override
                public void onCallback(User user, String error) {
                    RL_progress_bar.setVisibility(View.INVISIBLE);
                    if (error!=null){
                        //Si hay error lo mostramos
                        showMessages(error);
                    }else{
                        //Guardar el usuario
                        SharedPreference.getInstance(getApplicationContext()).setUserPreferences(user);
                        //Ir a la pantalla de reservas
                        goToBooking(user);
                    }
                }
            });
        }
        //Limpiamos la password
        ET_password.setText("");

    }
    private void checkIfUserInPreferences(){
        //Comprobamos si el usuario está en preferencias ya para no volver a pedirle el login
        User user = SharedPreference.getInstance(getApplicationContext()).getUserPreferences();
        if(user!=null){
            goToBooking(user);
        }
    }
    private void goToBooking(User user){
        Intent intent = new Intent(this, BookingActivity.class);
        intent.putExtra("user",user);
        startActivity(intent);
    }
    public void pressForgotPassword(){
        showMessages(getString(R.string.login_forgot_pass_alert));
    }


    private void showMessages( String string_to_show) {
        Toast.makeText(getApplicationContext(), string_to_show, Toast.LENGTH_SHORT).show();
    }

    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == 1) {
            if(resultCode == RESULT_OK) {
                User user = (User) data.getSerializableExtra("user");
                //Guardamos en preferencias
                SharedPreference.getInstance(getApplicationContext()).setUserPreferences(user);
                //Ir a la pantalla de reservas
                goToBooking(user);
            }
        }
    }

}
