/*
 * Copyright (c) 2019.
 * Autor: Beatriz Domínguez Callero
 * Esta obra está sujeta a una licencia de ReconocimientoNoComercial-SinObraDerivada 3.0 España de Creative Commons:
 * https://creativecommons.org/licenses/by-nc-nd/3.0/es/
 */

package com.beatriz.gestionaparcamiento.controller.activities;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RelativeLayout;
import android.widget.Toast;

import com.beatriz.gestionaparcamiento.R;
import com.beatriz.gestionaparcamiento.model.api.ApiCalls;
import com.beatriz.gestionaparcamiento.model.api.interfaces.RegisterCallBack;
import com.beatriz.gestionaparcamiento.model.objects.User;

import com.beatriz.gestionaparcamiento.controller.utils.Utils;
import com.beatriz.gestionaparcamiento.model.preferences.SharedPreference;

public class RegisterActivity extends AppCompatActivity {

    Button BT_register;
    EditText ET_email;
    EditText ET_password;
    EditText ET_password_repeat;
    RelativeLayout RL_progress_bar;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_register);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        //Inicialize element views:
        BT_register = findViewById(R.id.BT_register);
        ET_email = findViewById(R.id.ET_email);
        ET_password = findViewById(R.id.ET_password);
        ET_password_repeat = findViewById(R.id.ET_repeat_password);
        RL_progress_bar = findViewById(R.id.RL_progress_bar);

        BT_register.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                pressRegister();
            }
        });
    }

    public void pressRegister(){
        //Comprobamos si el email y la contraseña son válidos
        final String email = ET_email.getText().toString();
        final String password = ET_password.getText().toString();
        final String password_repeat = ET_password_repeat.getText().toString();
        if(email.isEmpty()){
            ET_email.setError(getString(R.string.register_error_email_empty));
        }else if(!Utils.isEmailValid(email)){
            ET_email.setError(getString(R.string.register_error_email_invalid));
        }else if(password.isEmpty()){
            ET_password.setError(getString(R.string.register_error_password_empty));
        }else if(password_repeat.isEmpty()) {
            ET_password_repeat.setError(getString(R.string.register_error_password_empty));
        }else if(!password.equals(password_repeat)){
            ET_password_repeat.setError(getString(R.string.register_error_password_not_equals));
        }else if(password.length()<6){
            ET_password_repeat.setError(getString(R.string.register_invalid_password_characters));
        }
        else{
            //Si son válidos, lanzamos la llamada al API
            RL_progress_bar.setVisibility(View.VISIBLE);
            ApiCalls.getInstance(getApplicationContext()).register(email, password, new RegisterCallBack() {
                @Override
                public void onCallback(User user, String error) {
                    RL_progress_bar.setVisibility(View.INVISIBLE);
                    if (error!=null){
                        //Si hay error lo mostramos
                        showMessages(error);
                    }else{
                        //si no hay error informamos al usuario
                        showAlert(getString(R.string.register_alert_title_ok),getString(R.string.register_alert_description_ok),getString(R.string.register_alert_ok_ok), user);
                    }
                }
            });
        }
        //Limpiamos las password
        ET_password.setText("");
        ET_password_repeat.setText("");

    }

    private void showMessages( String string_to_show) {
        Toast.makeText(getApplicationContext(), string_to_show, Toast.LENGTH_SHORT).show();
    }

    private void showAlert(String title, String message, String positiveButton, final User user){
        AlertDialog.Builder alert = new AlertDialog.Builder(this);
        alert.setTitle(title);
        alert.setMessage(message);
        alert.setPositiveButton(positiveButton, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                //Cuando el usuario se ha registrado correctamente, guardamos el usuario y forzamos ir a la pantalla de login para después ir a booking
                goToLoginAndBooking(user);
            }
        });
        alert.show();
    }

    private void goToLoginAndBooking(User user){
        Intent intent = new Intent();
        intent.putExtra("user",user);
        setResult(RESULT_OK, intent);
        finish();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        //Back button:
        if (id == android.R.id.home) {
            this.finish();
        }
        return super.onOptionsItemSelected(item);
    }
}


