/*
 * Copyright (c) 2019.
 * Autor: Beatriz Domínguez Callero
 * Esta obra está sujeta a una licencia de ReconocimientoNoComercial-SinObraDerivada 3.0 España de Creative Commons:
 * https://creativecommons.org/licenses/by-nc-nd/3.0/es/
 */

package com.beatriz.gestionaparcamiento.controller.fragments;

import androidx.recyclerview.widget.RecyclerView;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.beatriz.gestionaparcamiento.R;
import com.beatriz.gestionaparcamiento.model.objects.Parking;

import com.beatriz.gestionaparcamiento.controller.fragments.ParkingFreeFragment.OnListFragmentInteractionListener;

import java.util.ArrayList;
import java.util.List;

//Clase que gestiona el adapter de la lista de plazas libres
public class MyItemRecyclerViewAdapter extends RecyclerView.Adapter<MyItemRecyclerViewAdapter.ViewHolder> {

    private final List<Parking> mParkings;
    private final OnListFragmentInteractionListener mListener;
    private Context mContext;

    public MyItemRecyclerViewAdapter(ArrayList<Parking> items, Context context, OnListFragmentInteractionListener listener) {
        mParkings = items;
        mListener = listener;
        mContext = context;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.fragment_parkingfree_item, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(final ViewHolder holder, int position) {
        holder.mItem = mParkings.get(position);
        holder.mIdView.setText(mParkings.get(position).getParking_id().toString());
        if(mParkings.get(position).isAdmit_big()){
            holder.mContentView.setText(mContext.getString(R.string.parkingfree_header_admit_big_yes));
        }else{
            holder.mContentView.setText(mContext.getString(R.string.parkingfree_header_admit_big_no));
        }


        holder.mView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (null != mListener) {
                    // Notify the active callbacks interface (the activity, if the
                    // fragment is attached to one) that an item has been selected.
                    mListener.onListFragmentInteraction(holder.mItem);
                }
            }
        });
    }

    @Override
    public int getItemCount() {
        return mParkings.size();
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        public final View mView;
        public final TextView mIdView;
        public final TextView mContentView;
        public Parking mItem;

        public ViewHolder(View view) {
            super(view);
            mView = view;
            mIdView = (TextView) view.findViewById(R.id.item_number);
            mContentView = (TextView) view.findViewById(R.id.content);
        }

        @Override
        public String toString() {
            return super.toString() + " '" + mContentView.getText() + "'";
        }
    }
}
