/*
 * Copyright (c) 2019.
 * Autor: Beatriz Domínguez Callero
 * Esta obra está sujeta a una licencia de ReconocimientoNoComercial-SinObraDerivada 3.0 España de Creative Commons:
 * https://creativecommons.org/licenses/by-nc-nd/3.0/es/
 */

package com.beatriz.gestionaparcamiento.controller.fragments;

import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;

import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RelativeLayout;
import android.widget.Switch;
import android.widget.Toast;

import com.beatriz.gestionaparcamiento.R;
import com.beatriz.gestionaparcamiento.model.api.ApiCalls;
import com.beatriz.gestionaparcamiento.model.api.interfaces.SaveUserParkingCallBack;
import com.beatriz.gestionaparcamiento.model.objects.User;

import com.beatriz.gestionaparcamiento.controller.activities.BookingActivity;

public class MyParkingFragment extends Fragment {

    private Button BT_save;
    private EditText ET_num_parking;
    private Switch SW_admit_big;
    private User user;
    private static final String ARG_COLUMN_USER = "user";
    RelativeLayout RL_progress_bar;


    public MyParkingFragment(User user_init) {
        user = user_init;
    }


    public static MyParkingFragment newInstance(User user) {
        MyParkingFragment fragment = new MyParkingFragment(user);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Se inicializan los elementos
        View view = inflater.inflate(R.layout.fragment_my_parking, container, false);
        BT_save = view.findViewById(R.id.BT_save);
        ET_num_parking = view.findViewById(R.id.ET_n_parking);
        SW_admit_big = view.findViewById(R.id.SW_admit_big);
        RL_progress_bar = view.findViewById(R.id.RL_progress_bar);
        BT_save.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                pressSave();
            }
        });

        return view;
    }



    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
    }

    @Override
    public void onDetach() {
        super.onDetach();
    }

    private void pressSave(){
        //Controla la entrada del usuario y llama a guardar plaza cuando es necesario
        if(ET_num_parking.getText().toString().isEmpty()){
            ET_num_parking.setError(getString(R.string.myparking_error_parking_empty));
        }else{
            final Integer parking_id = Integer.parseInt(ET_num_parking.getText().toString());

            if(parking_id<=0){
                ET_num_parking.setError(getString(R.string.myparking_error_parking_invalid));
            }else {
                AlertDialog.Builder alert = new AlertDialog.Builder(getActivity());
                alert.setTitle(getString(R.string.myparking_alert_title_sure));
                alert.setMessage(getString(R.string.myparking_alert_description_sure)+" "+parking_id);
                alert.setPositiveButton(getString(R.string.myparking_alert_ok_sure), new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        saveParkingInServer(parking_id,SW_admit_big.isChecked());
                    }
                });
                alert.setNegativeButton(getString(R.string.myparking_alert_nook_sure), null);
                alert.show();
            }
        }
    }

    //Guarda la plaza del usuario y cierra el fragment
    private void saveParkingInServer(final Integer parking, final boolean admit_big){
        RL_progress_bar.setVisibility(View.VISIBLE);
        ApiCalls.getInstance(getContext()).save_parking(parking, admit_big, new SaveUserParkingCallBack() {
            @Override
            public void onCallback(String error) {
                if(error!=null){
                    RL_progress_bar.setVisibility(View.INVISIBLE);
                    showMessages(error);
                }else{
                    ApiCalls.getInstance(getContext()).save_user_parking(user.getUser_id(), parking, new SaveUserParkingCallBack() {
                        @Override
                        public void onCallback(String error) {
                            if(error!=null){
                                RL_progress_bar.setVisibility(View.INVISIBLE);
                                showMessages(error);
                            }else{
                                RL_progress_bar.setVisibility(View.INVISIBLE);
                                showMessages(getString(R.string.myparking_save_ok));
                                user.setParking_id(parking);
                                ((BookingActivity) getActivity()).saveUserWithParkingAndCloseFragment(user, admit_big);
                            }
                        }
                    });
                }
            }
        });

    }

    private void showMessages( String string_to_show) {
        Toast.makeText(getContext(), string_to_show, Toast.LENGTH_SHORT).show();
    }

}
