/*
 * Copyright (c) 2019.
 * Autor: Beatriz Domínguez Callero
 * Esta obra está sujeta a una licencia de ReconocimientoNoComercial-SinObraDerivada 3.0 España de Creative Commons:
 * https://creativecommons.org/licenses/by-nc-nd/3.0/es/
 */

package com.beatriz.gestionaparcamiento.controller.fragments;
import android.content.Context;
import android.os.Bundle;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.Toast;

import com.beatriz.gestionaparcamiento.R;
import com.beatriz.gestionaparcamiento.model.api.ApiCalls;
import com.beatriz.gestionaparcamiento.model.api.interfaces.DeleteReleaseParkingCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.GetFreeParkingsCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.SaveReleaseParkingCallBack;
import com.beatriz.gestionaparcamiento.model.objects.Parking;
import com.beatriz.gestionaparcamiento.model.objects.User;

import java.util.ArrayList;

import com.beatriz.gestionaparcamiento.controller.activities.BookingActivity;


public class ParkingFreeFragment extends Fragment {

    private String day;
    private User user;
    private OnListFragmentInteractionListener mListener;
    Context context;
    RecyclerView recyclerView;
    ArrayList<Parking> parkings_free;
    RelativeLayout RL_progress_bar;
    ImageButton BT_cancel;
    LinearLayout LL_layout_list;
    LinearLayout LL_layout_empty;


    public ParkingFreeFragment(String day_param, User user_param) {
        day=day_param;
        user=user_param;
    }


    public static ParkingFreeFragment newInstance(String day, User user) {
        ParkingFreeFragment fragment = new ParkingFreeFragment(day,user);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_parkingfree, container, false);

        //Inicializamos los elementos
        context = view.getContext();
        recyclerView= (RecyclerView) view.findViewById(R.id.parking_free_fragment_list);
        recyclerView.setLayoutManager(new LinearLayoutManager(context));
        RL_progress_bar = view.findViewById(R.id.RL_progress_bar);
        BT_cancel = view.findViewById(R.id.BT_cancel);
        LL_layout_empty = view.findViewById(R.id.LL_emptyList);
        LL_layout_list = view.findViewById(R.id.LL_list_layouts);
        //Cuando se pulsa X se sale del fragment
        BT_cancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                ((BookingActivity) getActivity()).closeFreeFragment(false);
            }
        });
        //Creamos el listener: cuando el usuario pulsa un elemento, se almacena la información de la reserva y se cierra el fragment.
        mListener = new OnListFragmentInteractionListener() {
            @Override
            public void onListFragmentInteraction(final Parking item) {
                RL_progress_bar.setVisibility(View.VISIBLE);
                ApiCalls.getInstance(context).delete_release_parking(day, item.getParking_id(), new DeleteReleaseParkingCallBack() {
                    @Override
                    public void onCallback(String error) {
                        RL_progress_bar.setVisibility(View.INVISIBLE);
                        if(error!=null){
                            showMessages(error);
                        }else{
                            ApiCalls.getInstance(context).save_booking(day, user, item, new SaveReleaseParkingCallBack() {
                                @Override
                                public void onCallback(String error) {
                                    if(error!=null){
                                        showMessages(error);
                                    }else{
                                        ((BookingActivity) getActivity()).closeFreeFragment(true);
                                    }
                                }
                            });
                        }

                    }
                });

            }
        };

        getFreeParkingAndSetAdapter();
        return view;
    }


    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }


    public interface OnListFragmentInteractionListener {
        void onListFragmentInteraction(Parking item);
    }

    private void getFreeParkingAndSetAdapter(){
        //Obtiene la información de las plazas libres.
        //Si no hay plazas, se muestra un mensaje, si las hay, la lista
        RL_progress_bar.setVisibility(View.VISIBLE);
        ApiCalls.getInstance(context).get_free_parking(day, new GetFreeParkingsCallBack() {
            @Override
            public void onCallback(ArrayList<Parking> value, String error) {
                RL_progress_bar.setVisibility(View.INVISIBLE);
                parkings_free = value;
                if(parkings_free.isEmpty()){
                    LL_layout_empty.setVisibility(View.VISIBLE);
                    LL_layout_list.setVisibility(View.GONE);
                }else{
                    LL_layout_empty.setVisibility(View.GONE);
                    LL_layout_list.setVisibility(View.VISIBLE);
                    recyclerView.setAdapter(new MyItemRecyclerViewAdapter(parkings_free,context, mListener));
                }

            }
        });
    }
    private void showMessages( String string_to_show) {
        Toast.makeText(context, string_to_show, Toast.LENGTH_SHORT).show();
    }
}
