/*
 * Copyright (c) 2019.
 * Autor: Beatriz Domínguez Callero
 * Esta obra está sujeta a una licencia de ReconocimientoNoComercial-SinObraDerivada 3.0 España de Creative Commons:
 * https://creativecommons.org/licenses/by-nc-nd/3.0/es/
 */

package com.beatriz.gestionaparcamiento.model.api;

/*Esta clase se encarga de realizar las llamadas al API y transformar las respuestas a objetos que usa la aplicación*/

import android.app.Activity;
import android.content.Context;
import android.util.Log;

import androidx.annotation.NonNull;

import com.beatriz.gestionaparcamiento.R;
import com.beatriz.gestionaparcamiento.model.api.interfaces.DeleteBookingParkingCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.DeleteReleaseParkingCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.GetBookingCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.GetFreeParkingsCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.GetInfoReleaseCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.GetParkingCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.GetUserParkingCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.LoginCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.RegisterCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.SaveReleaseParkingCallBack;
import com.beatriz.gestionaparcamiento.model.api.interfaces.SaveUserParkingCallBack;
import com.beatriz.gestionaparcamiento.model.objects.Booking;
import com.beatriz.gestionaparcamiento.model.objects.Parking;
import com.beatriz.gestionaparcamiento.model.objects.User;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public class ApiCalls extends Activity{

    private static ApiCalls mInstance = null;

    FirebaseAuth mAuth;
    DatabaseReference mReference;
    FirebaseDatabase mDatabase;
    Context mContext;

    private ApiCalls(Context context){
        mAuth = FirebaseAuth.getInstance();
        mDatabase = FirebaseDatabase.getInstance();
        mReference=mDatabase.getReference();
        mContext=context;
    }
    public static ApiCalls getInstance(Context context) {
        if (mInstance == null) {
            mInstance = new ApiCalls(context);
        }
        return mInstance;
    }

    private String translateExceptionIfIsPosible(String message){
        switch (message){
            case "The email address is already in use by another account.":
                return mContext.getString(R.string.email_in_use);
            case "The password is invalid or the user does not have a password.":
                return mContext.getString(R.string.invalid_password_or_user);
            case "There is no user record corresponding to this identifier. The user may have been deleted.":
                return mContext.getString(R.string.invalid_password_or_user);
            case "The given password is invalid. [ Password should be at least 6 characters ]":
                return mContext.getString(R.string.invalid_password_characters);
            case "An internal error has occurred. [ 7: ]":
                return mContext.getString(R.string.check_connection);
            default:
                return message;
        }

    }


    /*1. Registro del usuario en Firebase*/
    public void register(final String email, String password, final RegisterCallBack registerCallBack){
        mAuth.createUserWithEmailAndPassword(email, password)
                .addOnCompleteListener(this, new OnCompleteListener<AuthResult>() {
                @Override
                public void onComplete(@NonNull Task<AuthResult> task) {
                    if (task.isSuccessful()) {
                        // Sign in success, update UI with the signed-in user's information
                        Log.d("ok", "createUserWithEmail:success");
                        FirebaseUser user = mAuth.getCurrentUser();
                        User user_to_return= new User();
                        user_to_return.setUser_id(user.getUid());
                        user_to_return.setEmail(email);
                        registerCallBack.onCallback(user_to_return,null);

                    } else {
                        // If sign in fails, display a message to the user.
                        Log.w("ApiCallError", "register:failure", task.getException());
                        registerCallBack.onCallback(null,translateExceptionIfIsPosible(task.getException().getMessage()));
                    }
                }
        });
    }

    /*2. Login del usuario en Firebase*/
    public void login(final String email, String password, final LoginCallBack loginCallBack){
        mAuth.signInWithEmailAndPassword(email, password)
                .addOnCompleteListener(this, new OnCompleteListener<AuthResult>() {
                    @Override
                    public void onComplete(@NonNull Task<AuthResult> task) {
                        if (task.isSuccessful()) {
                            // Sign in success, update UI with the signed-in user's information
                            Log.d("ok", "singIn:success");
                            FirebaseUser user = mAuth.getCurrentUser();
                            User user_to_return= new User();
                            user_to_return.setUser_id(user.getUid());
                            user_to_return.setEmail(email);
                            loginCallBack.onCallback(user_to_return,null);
                        } else {
                            // If sign in fails, display a message to the user.
                            Log.w("ApiCallError", "login:failure", task.getException());
                            loginCallBack.onCallback(null,translateExceptionIfIsPosible(task.getException().getLocalizedMessage()));
                        }
                    }
                });
    }

    /*3. Obtener la plaza de un determinado usuario*/
    public void get_user_parking(String user_id, final GetUserParkingCallBack getUserParkingCallBack){
        mReference.child("user").child(user_id).child("parking_id")
            .addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                    Log.d("ok", "singIn:success");
                    if(dataSnapshot.getValue()==null){
                        getUserParkingCallBack.onCallback(null,null);
                    }else{
                        Integer number_parking = Integer.parseInt(dataSnapshot.getValue().toString());
                        getUserParkingCallBack.onCallback(number_parking, null);
                    }
                }

                @Override
                public void onCancelled(@NonNull DatabaseError databaseError) {
                    // If sign in fails, display a message to the user.
                    Log.w("ApiCallError", "get_user_parking:failure",databaseError.toException());
                    getUserParkingCallBack.onCallback(null,translateExceptionIfIsPosible(databaseError.toException().getMessage()));
                }});
    }

    /*4. Obtener la información de una plaza.*/
    public void get_parking(Integer parking_id, final GetParkingCallBack getParkingCallBack){
        mReference.child("parking").child(parking_id.toString())
                .addListenerForSingleValueEvent(new ValueEventListener() {
                    @Override
                    public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                        Log.d("ok", "singIn:success");
                        //Rellenar luego
                        Parking parking = new Parking();
                        parking.setParking_id(Integer.parseInt(dataSnapshot.getKey()));
                        parking.setAdmit_big((Boolean)((HashMap) dataSnapshot.getValue()).get("admit_big"));
                        getParkingCallBack.onCallback(parking,null);
                    }

                    @Override
                    public void onCancelled(@NonNull DatabaseError databaseError) {
                        // If sign in fails, display a message to the user.
                        Log.w("ApiCallError", "get_parking:failure",databaseError.toException());
                        getParkingCallBack.onCallback(null,translateExceptionIfIsPosible(databaseError.toException().getMessage()));
                    }});
    }


    /*5. Guardar la información de la plaza de un determinado usuario.*/
    public void save_user_parking(String user_id, Integer parking_id, final SaveUserParkingCallBack saveUserParkingCallBack){
        mReference.child("user") .child(user_id).child("parking_id").setValue(parking_id)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void aVoid) {
                        saveUserParkingCallBack.onCallback(null);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.w("ApiCallError", "save_user_parking:failure",e);
                        saveUserParkingCallBack.onCallback(translateExceptionIfIsPosible(e.getMessage()));
                    }
                });
    }

    /*6. Guardar información de una plaza-*/
    public void save_parking(Integer parking_id, Boolean admit_big, final SaveUserParkingCallBack saveParkingCallBack){
        mReference.child("parking").child(parking_id.toString()).child("admit_big").setValue(admit_big)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void aVoid) {
                        saveParkingCallBack.onCallback(null);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.w("ApiCallError", "save_user_parking:failure",e);
                        saveParkingCallBack.onCallback(translateExceptionIfIsPosible(e.getMessage()));
                    }
                });
    }

    /*7. Saber si la plaza de un usuario ha sido liberada o reservada por otro*/
    /* Si se devuelve null --> la plaza del usuario no ha sido liberada para este dia
        Si devuelve Booking --> la plaza ha sido liberada
            Si Booking.user == null --> La plaza nunca no ha sido liberada por otro usuario
            Si Booking.user != null --> La plaza ha sido liberada por otro usuario*/
    public void get_info_release(String day, Integer parking_id, final GetInfoReleaseCallBack getInfoReleaseCallBack) {
        mReference.child("booking").child(day).orderByChild("parking_id").equalTo(parking_id)
                .addListenerForSingleValueEvent(new ValueEventListener() {
                    @Override
                    public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                        Log.d("ok", "singIn:success");
                        HashMap data = (HashMap) dataSnapshot.getValue();
                        if (data==null){
                            //La plaza del usuario no ha sido liberada
                            getInfoReleaseCallBack.onCallback(null,null);
                        }else{
                            data = (HashMap) data.get(data.keySet().toArray()[0]);
                            Booking booking = new Booking();
                            booking.setUser_id((String) data.get("user_id"));
                            booking.setParkingId(Integer.parseInt(data.get("parking_id").toString()));
                            booking.setAdmit_big((Boolean) data.get("admit_big"));
                            getInfoReleaseCallBack.onCallback(booking,null);
                        }


                    }

                    @Override
                    public void onCancelled(@NonNull DatabaseError databaseError) {
                        // If sign in fails, display a message to the user.
                        Log.w("ApiCallError", "get_parking:failure",databaseError.toException());
                        getInfoReleaseCallBack.onCallback(null,translateExceptionIfIsPosible(databaseError.toException().getMessage()));
                    }});
    }

    /*8. Obtener información de las reservas de un usuario para un determinado día*/
    /*Si se devuelve null --> No tiene reservas ese día
        Si devuelve Booking --> Tiene reserva ese dia*/
    public void get_booking(String day, String user_id, final GetBookingCallBack getBookingCallBack) {
        mReference.child("booking").child(day).orderByChild("user_id").equalTo(user_id)
                .addListenerForSingleValueEvent(new ValueEventListener() {
                    @Override
                    public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                        Log.d("ok", "singIn:success");
                        HashMap data = (HashMap) dataSnapshot.getValue();
                        if (data==null){
                            getBookingCallBack.onCallback(null,null);
                        }else{
                            data = (HashMap) data.get(data.keySet().toArray()[0]);
                            Booking booking = new Booking();
                            booking.setUser_id((String) data.get("user_id"));
                            booking.setParkingId(Integer.parseInt(data.get("parking_id").toString()));
                            booking.setAdmit_big((Boolean) data.get("admit_big"));
                            getBookingCallBack.onCallback(booking,null);
                        }


                    }

                    @Override
                    public void onCancelled(@NonNull DatabaseError databaseError) {
                        // If sign in fails, display a message to the user.
                        Log.w("ApiCallError", "get_parking:failure",databaseError.toException());
                        getBookingCallBack.onCallback(null,translateExceptionIfIsPosible(databaseError.toException().getMessage()));
                    }});
    }

    /*9. Obtener plazas disponibles
Descripción Se obtienen las plazas disponibles un determinado día*/
    public void get_free_parking(String day, final GetFreeParkingsCallBack getFreeParkingsCallBack) {
        mReference.child("booking").child(day).orderByChild("user_id").equalTo(null)
                .addListenerForSingleValueEvent(new ValueEventListener() {
                    ArrayList<Parking> parkings = new ArrayList<>();
                    @Override
                    public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                        Log.d("ok", "singIn:success");
                        HashMap data = (HashMap) dataSnapshot.getValue();
                        if (data==null){
                            getFreeParkingsCallBack.onCallback(parkings,null);
                        }else{
                            final Iterator <Map.Entry<String, HashMap>> it = data.entrySet().iterator();
                            Map.Entry<String, HashMap> entry= null;
                            while (it.hasNext()) {
                                entry= it.next();
                                HashMap parking = entry.getValue();
                                Parking parking_return = new Parking();
                                parking_return.setParking_id(Integer.parseInt((parking.get("parking_id").toString())));
                                parking_return.setAdmit_big((Boolean) parking.get("admit_big"));
                                parkings.add(parking_return);
                            }
                            getFreeParkingsCallBack.onCallback(parkings,null);
                        }


                    }

                    @Override
                    public void onCancelled(@NonNull DatabaseError databaseError) {
                        // If sign in fails, display a message to the user.
                        Log.w("ApiCallError", "get_parking:failure",databaseError.toException());
                        getFreeParkingsCallBack.onCallback(null,translateExceptionIfIsPosible(databaseError.toException().getMessage()));
                    }});
    }


    /*10. Liberar una plaza un determinado día */
    public void release_parking(String day, Parking parking, final SaveReleaseParkingCallBack saveReleaseParkingCallBack){
        String key_booking = "null_"+parking.getParking_id();
        HashMap booking_hasmap= new HashMap();
        booking_hasmap.put("parking_id",parking.getParking_id());
        booking_hasmap.put("admit_big",parking.isAdmit_big());
        booking_hasmap.put("user_id",null);
        mReference.child("booking").child(day).child(key_booking).setValue(booking_hasmap)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void aVoid) {
                        saveReleaseParkingCallBack.onCallback(null);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.w("ApiCallError", "save_user_parking:failure",e);
                        saveReleaseParkingCallBack.onCallback(translateExceptionIfIsPosible(e.getMessage()));
                    }
                });
    }


    /*11. Eliminar la liberación de la plaza un determinado día*/
    public void delete_release_parking(String day, Integer parking_id, final DeleteReleaseParkingCallBack deleteReleaseParkingCallBack){
        String key = "null_"+parking_id;
        mReference.child("booking").child(day).child(key).removeValue()
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void aVoid) {
                        deleteReleaseParkingCallBack.onCallback(null);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.w("ApiCallError", "save_user_parking:failure",e);
                        deleteReleaseParkingCallBack.onCallback(translateExceptionIfIsPosible(e.getMessage()));
                    }
                });
    }

    /*12. Reservar una plaza un determinado día*/
    public void save_booking(String day, User user, Parking parking, final SaveReleaseParkingCallBack saveReleaseParkingCallBack){
        String key_booking = user.getUser_id()+"_"+parking.getParking_id();
        HashMap booking_hasmap= new HashMap();
        booking_hasmap.put("parking_id",parking.getParking_id());
        booking_hasmap.put("admit_big",parking.isAdmit_big());
        booking_hasmap.put("user_id",user.getUser_id());
        mReference.child("booking").child(day).child(key_booking).setValue(booking_hasmap)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void aVoid) {
                        saveReleaseParkingCallBack.onCallback(null);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.w("ApiCallError", "save_user_parking:failure",e);
                        saveReleaseParkingCallBack.onCallback(translateExceptionIfIsPosible(e.getMessage()));
                    }
                });
    }

    /*13. Liberar una plaza un determinado día - Eliminar reserva en base de datos*/
    public void delete_booking_parking(String day, String user_id, Integer parking_id, final DeleteBookingParkingCallBack deleteBookingParkingCallBack){
        String key = user_id+"_"+parking_id;
        mReference.child("booking").child(day).child(key).removeValue()
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void aVoid) {
                        deleteBookingParkingCallBack.onCallback(null);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.w("ApiCallError", "save_user_parking:failure",e);
                        deleteBookingParkingCallBack.onCallback(translateExceptionIfIsPosible(e.getMessage()));
                    }
                });
    }

}
